from django.shortcuts import render_to_response

try:
    from django.conf.urls import patterns, include, url
except ImportError:
    from django.conf.urls.defaults import patterns, include, url

from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
                       url(r'^$', lambda r: render_to_response('index.html'), name='index'),
                       url(r'^adm1n/', include(admin.site.urls)))



import cio
import os
import simplejson as json
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test import Client
from django.utils.http import urlquote
from cio.plugins import plugins
from cio.backends import storage
from cio.backends.exceptions import PersistenceError, NodeDoesNotExist
from cio.utils.uri import URI
from djedi.tests.base import DjediTest, UserMixin, ClientTest
from djedi.utils.encoding import smart_unicode


def json_node(response, simple=True):
    node = json.loads(response.content)
    if simple and 'meta' in node:
        del node['meta']
    return node


class PermissionTest(DjediTest, UserMixin):

    def setUp(self):
        super(PermissionTest, self).setUp()
        self.master = self.create_djedi_master()
        self.apprentice = self.create_djedi_apprentice()

    def test_permissions(self):
        client = Client()
        url = reverse('admin:djedi:api', args=['i18n://sv-se@page/title'])

        response = client.get(url)
        self.assertEqual(response.status_code, 403)

        logged_in = client.login(username=self.master.username, password='test')
        self.assertTrue(logged_in)
        response = client.get(url)
        self.assertEqual(response.status_code, 404)

        client.logout()
        logged_in = client.login(username=self.apprentice.username, password='test')
        self.assertTrue(logged_in)
        response = client.get(url)
        self.assertEqual(response.status_code, 404)


class RestTest(ClientTest):

    def get_api_url(self, url_name, uri):
        return reverse('admin:djedi:' + url_name, args=[urlquote(urlquote(uri, ''), '')])

    def get(self, url_name, uri):
        url = self.get_api_url(url_name, uri)
        return self.client.get(url)

    def post(self, url_name, uri, data):
        url = self.get_api_url(url_name, uri)
        return self.client.post(url, data)

    def put(self, url_name, uri, data=None):
        url = self.get_api_url(url_name, uri)
        return self.client.put(url, data=data or {})

    def delete(self, url_name, uri):
        url = self.get_api_url(url_name, uri)
        return self.client.delete(url)

    def test_get(self):
        response = self.get('api', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 404)

        cio.set('i18n://sv-se@page/title.md', u'# Djedi', publish=False)

        response = self.get('api', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 404)

        response = self.get('api', 'i18n://sv-se@page/title#draft')
        self.assertEqual(response.status_code, 200)
        node = json_node(response)
        self.assertKeys(node, 'uri', 'content')
        self.assertEqual(node['uri'], 'i18n://sv-se@page/title.md#draft')
        self.assertRenderedMarkdown(node['content'], u'# Djedi')

    def test_load(self):
        response = self.get('api.load', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 200)
        json_content = json.loads(response.content)
        self.assertEqual(json_content['uri'], 'i18n://sv-se@page/title.txt')
        self.assertIsNone(json_content['data'])
        self.assertEqual(len(json_content['meta'].keys()), 0)

        # TODO: Should get 404
        # response = self.get('api.load', 'i18n://sv-se@page/title#1')
        # self.assertEqual(response.status_code, 404)

        cio.set('i18n://sv-se@page/title.md', u'# Djedi')

        response = self.get('api.load', 'sv-se@page/title')
        self.assertEqual(response.status_code, 200)
        node = json_node(response, simple=False)
        meta = node.pop('meta', {})
        content = u'# Djedi' if cio.PY26 else u'<h1>Djedi</h1>'
        self.assertDictEqual(node, {'uri': 'i18n://sv-se@page/title.md#1', 'data': u'# Djedi', 'content': content})
        self.assertKeys(meta, 'modified_at', 'published_at', 'is_published')

        response = self.get('api.load', 'i18n://sv-se@page/title#1')
        json_content = json.loads(response.content)
        self.assertEqual(json_content['uri'], 'i18n://sv-se@page/title.md#1')

        self.assertEqual(len(cio.revisions('i18n://sv-se@page/title')), 1)

    def test_set(self):
        response = self.post('api', 'i18n://page/title', {'data': u'# Djedi'})
        self.assertEqual(response.status_code, 400)

        response = self.post('api', 'i18n://sv-se@page/title.txt', {'data': u'# Djedi', 'data[extra]': u'foobar'})
        self.assertEqual(response.status_code, 400)

        uri = 'i18n://sv-se@page/title.md'
        response = self.post('api', uri, {'data': u'# Djedi', 'meta[message]': u'lundberg'})
        self.assertEqual(response.status_code, 200)
        node = json_node(response, simple=False)
        meta = node.pop('meta')
        content = u'# Djedi' if cio.PY26 else u'<h1>Djedi</h1>'
        self.assertDictEqual(node, {'uri': 'i18n://sv-se@page/title.md#draft', 'content': content})
        self.assertEqual(meta['author'], u'master')
        self.assertEqual(meta['message'], u'lundberg')

        node = cio.get(uri, lazy=False)
        self.assertIsNone(node.content)
        cio.publish(uri)
        node = cio.get(uri, lazy=False)
        self.assertEqual(node.uri, 'i18n://sv-se@page/title.md#1')
        self.assertRenderedMarkdown(node.content, u'# Djedi')

        response = self.post('api', node.uri, {'data': u'# Djedi', 'meta[message]': u'Lundberg'})
        node = json_node(response, simple=False)
        self.assertEqual(node['meta']['message'], u'Lundberg')

        with self.assertRaises(PersistenceError):
            storage.backend._create(URI(node['uri']), None)

    def test_delete(self):
        response = self.delete('api', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 404)

        node = cio.set('i18n://sv-se@page/title.md', u'# Djedi')

        response = self.delete('api', node.uri)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_unicode(response.content), u'')

        with self.assertRaises(NodeDoesNotExist):
            storage.get('i18n://sv-se@page/title')

        node = cio.get('i18n://page/title', lazy=False)
        self.assertIsNone(node.content)

    def test_publish(self):
        node = cio.set('sv-se@page/title', u'Djedi', publish=False)

        response = self.get('api', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 404)

        response = self.put('api.publish', node.uri)
        self.assertEqual(response.status_code, 200)

        response = self.get('api', 'i18n://sv-se@page/title')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_node(response), {'uri': 'i18n://sv-se@page/title.txt#1', 'content': u'Djedi'})
        response = self.put('api.publish', 'i18n://sv-se@foo/bar.txt#draft')

        self.assertEqual(response.status_code, 404)

    def test_revisions(self):
        cio.set('sv-se@page/title', u'Djedi 1')
        cio.set('sv-se@page/title', u'Djedi 2')

        response = self.get('api.revisions', 'sv-se@page/title')
        self.assertEqual(response.status_code, 200)

        content = json.loads(response.content)
        self.assertEqual(content, [['i18n://sv-se@page/title.txt#1', False], ['i18n://sv-se@page/title.txt#2', True]])

    def test_render(self):
        response = self.post('api.render', 'foo', {'data': u'# Djedi'})
        assert response.status_code == 404

        response = self.post('api.render', 'md', {'data': u'# Djedi'})
        assert response.status_code == 200
        self.assertRenderedMarkdown(smart_unicode(response.content), u'# Djedi')

        response = self.post('api.render', 'img', {'data': json.dumps({
            'url': '/foo/bar.png',
            'width': '64',
            'height': '64'
        })})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(smart_unicode(response.content), u'<img height="64" src="/foo/bar.png" width="64" />')

    def test_editor(self):
        response = self.get('cms.editor', 'sv-se@page/title.foo')
        self.assertEqual(response.status_code, 404)

        response = self.get('cms.editor', 'sv-se@page/title')
        self.assertEqual(response.status_code, 404)

        for ext in plugins:
            response = self.get('cms.editor', 'sv-se@page/title.' + ext)
            self.assertEqual(response.status_code, 200)
            assert set(response.context_data.keys()) == set(('THEME', 'VERSION', 'uri',))

        response = self.post('cms.editor', 'sv-se@page/title', {'data': u'Djedi'})
        self.assertEqual(response.status_code, 200)

    def test_upload(self):
        tests_dir = os.path.dirname(os.path.abspath(__file__))
        image_path = os.path.join(tests_dir, 'assets', 'image.png')

        form = {
            'data[width]': u'64',
            'data[height]': u'64',
            'data[crop]': u'64,64,128,128',
            'data[id]': u'vw',
            'data[class]': u'year-53',
            'data[alt]': u'Zwitter',
            'meta[comment]': u'VW'
        }
        response = self.post('api', 'i18n://sv-se@header/logo.img', form)
        self.assertEqual(response.status_code, 200)

        with open(image_path, "rb") as image:
            file = File(image, name=image_path)
            form['data[file]'] = file
            response = self.post('api', 'i18n://sv-se@header/logo.img', form)
            self.assertEqual(response.status_code, 200)

            node = json_node(response, simple=False)
            meta = node.pop('meta')
            uri, content = node['uri'], node['content']
            self.assertEqual(uri, 'i18n://sv-se@header/logo.img#draft')
            self.assertEqual(meta['comment'], u'VW')
            html = u'<img ' \
                   u'alt="Zwitter" ' \
                   u'class="year-53" ' \
                   u'height="64" ' \
                   u'id="vw" ' \
                   u'src="/media/djedi/img/03/5e/5eba6fc2149822a8dbf76cd6978798f2ddc4ac34.png" ' \
                   u'width="64" />'
            self.assertEqual(content, html)

            # Post new resized version
            node = cio.load(uri)
            del form['data[file]']
            del form['data[crop]']
            form['data[width]'] = form['data[height]'] = u'32'
            form['data[filename]'] = node['data']['filename']

            response = self.post('api', 'i18n://sv-se@header/logo.img', form)
            self.assertEqual(response.status_code, 200)



try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url

from .api import NodeApi, LoadApi, PublishApi, RevisionsApi, RenderApi, NodeEditor
from .cms import DjediCMS

urlpatterns = patterns(
    '',
    url(r'^$', DjediCMS.as_view(), name='cms'),
    url(r'^node/(?P<uri>.+)/editor$', NodeEditor.as_view(), name='cms.editor'),
    url(r'^node/(?P<uri>.+)/load$', LoadApi.as_view(), name='api.load'),
    url(r'^node/(?P<uri>.+)/publish$', PublishApi.as_view(), name='api.publish'),
    url(r'^node/(?P<uri>.+)/revisions$', RevisionsApi.as_view(), name='api.revisions'),
    url(r'^node/(?P<uri>.+)$', NodeApi.as_view(), name='api'),
    url(r'^plugin/(?P<ext>\w+)$', RenderApi.as_view(), name='api.render'),
)



import json
import logging

from django import template

from cio.pipeline import pipeline
from djedi.auth import has_permission
from djedi.compat import render_to_string


register = template.Library()
logger = logging.getLogger(__name__)


@register.simple_tag(takes_context=True)
def djedi_admin(context):
    output = u''

    if has_permission(context.get('user')):
        defaults = dict((node.uri.clone(version=None), node.initial) for node in pipeline.history.list('get'))
        output = render_to_string('djedi/cms/embed.html', {
            'json_nodes': json.dumps(defaults).replace('</', '\\x3C/'),
        })

    # Clear pipeline
    pipeline.clear()

    return output



import shutil

import cio
import django

from contextlib import contextmanager

from django.conf import settings
from django.contrib.auth.models import User, Group
from django.test import Client
from django.test import TransactionTestCase


if django.VERSION < (1, 8):
    DEBUG_CURSOR_ATTR = 'use_debug_cursor'
else:
    DEBUG_CURSOR_ATTR = 'force_debug_cursor'


class DjediTest(TransactionTestCase):

    def setUp(self):
        from cio.environment import env
        from cio.backends import cache
        from cio.pipeline import pipeline
        from cio.plugins import plugins

        env.reset()
        cache.clear()
        pipeline.clear()
        plugins.load()

    def tearDown(self):
        shutil.rmtree(settings.MEDIA_ROOT, ignore_errors=True)


class AssertionMixin(object):

    def assertKeys(self, dict, *keys):
        self.assertEqual(set(dict.keys()), set(keys))

    @contextmanager
    def assertCache(self, calls=-1, hits=-1, misses=-1):
        from cio.backends import cache

        _cache = cache.backend._cache

        _cache.calls = 0
        _cache.hits = 0
        _cache.misses = 0

        yield

        if calls >= 0:
            assert _cache.calls == calls
        if hits >= 0:
            assert _cache.hits == hits
        if misses >= 0:
            assert _cache.misses == misses

    @contextmanager
    def assertDB(self, calls=-1, selects=-1, inserts=-1, updates=-1):
        from django.db import connection

        pre_debug_cursor = getattr(connection, DEBUG_CURSOR_ATTR)
        setattr(connection, DEBUG_CURSOR_ATTR, True)
        pre_num_queries = len(connection.queries)

        yield

        queries = connection.queries[pre_num_queries:]
        num_queries = len(queries)
        setattr(connection, DEBUG_CURSOR_ATTR, pre_debug_cursor)

        if calls >= 0:
            assert num_queries == calls
        if selects >= 0:
            num_selects = len([q for q in queries if q['sql'].startswith('SELECT')])
            assert num_selects == selects
        if inserts >= 0:
            num_inserts = len([q for q in queries if q['sql'].startswith('INSERT')])
            assert num_inserts == inserts
        if updates >= 0:
            num_updates = len([q for q in queries if q['sql'].startswith('UPDATE')])
            assert num_updates == updates

    def assertRenderedMarkdown(self, value, source):
        if cio.PY26:
            self.assertEqual(value, source)  # Markdown lacks Support for python 2.6
        else:
            from markdown import markdown
            rendered = markdown(source)
            self.assertEqual(value, rendered)


class UserMixin(object):

    def create_djedi_master(self):
        user = User.objects.create_superuser('master', 'master@djedi.io', 'test')
        return user

    def create_djedi_apprentice(self):
        user = User.objects.create_user('apprentice', email='apprentice@djedi.io', password='test')
        group, _ = Group.objects.get_or_create(name='Djedi')
        user.is_staff = True
        user.groups.add(group)
        user.save()
        return user


class ClientTest(DjediTest, UserMixin, AssertionMixin):

    def setUp(self):
        super(ClientTest, self).setUp()
        master = self.create_djedi_master()
        client = Client(enforce_csrf_checks=True)
        client.login(username=master.username, password='test')
        self.client = client



from . import DjediMiddleware
from .mixins import AdminPanelMixin


class DjediAdminMiddleware(DjediMiddleware, AdminPanelMixin):

    def process_response(self, request, response):
        self.inject_admin_panel(request, response)
        return super(DjediAdminMiddleware, self).process_response(request, response)



import json
import six
from django.utils.html import escape
from cio.plugins.base import BasePlugin
from django.core.files.uploadedfile import InMemoryUploadedFile
from hashlib import sha1
from os import path


class ImagePluginBase(BasePlugin):

    ext = 'img'

    def _open(self, filename):
        raise NotImplementedError

    def _save(self, filename, bytes):
        raise NotImplementedError

    def _url(self, filename):
        raise NotImplementedError

    def _create_filename(self, filename, **kwargs):
        name, ext = path.splitext(filename)
        dir, name = name.rsplit(path.sep, 1)
        name += ''.join(sorted(key + str(value) for key, value in six.iteritems(kwargs)))

        if six.PY3:
            name = name.encode('utf-8')

        name = sha1(name).hexdigest()
        subdir = name[:2]
        return path.sep.join((dir, subdir, name + ext))

    def load(self, content):
        if content:
            data = json.loads(content)

            # Add image url to loaded data
            filename = data.get('filename', None)
            if filename:
                data['url'] = self._url(filename)

            return data
        else:
            return {'filename': None, 'url': None}

    def save(self, data):
        from PIL import Image
        width = int(data.get('width') or 0)
        height = int(data.get('height') or 0)

        file = None
        upload = data.get('file')
        filename = data.get('filename')

        if upload:
            image = Image.open(upload)
            filename = path.sep.join(('djedi', 'img', upload.name))
            filename = self._create_filename(filename, w=width, h=height)
        elif filename:
            file = self._open(filename)
            image = Image.open(file)
        else:
            image = None

        if image:
            # Remember original image format before processing
            format = image.format

            # Crop
            crop = data.get('crop')
            if crop:
                try:
                    box = tuple(int(x) for x in crop.split(','))
                    image = image.crop(box)
                except:
                    pass  # TODO: Handle image crop error
                else:
                    filename = self._create_filename(filename, crop=crop)

            # Resize
            i_width, i_height = image.size
            if (width and width != i_width) or (height and height != i_height):
                try:
                    image = image.resize((width, height), Image.ANTIALIAS)
                except:
                    pass
                else:
                    filename = self._create_filename(filename, w=width, h=height)
            else:
                width = i_width
                height = i_height

            # Write file
            if filename != data.get('filename'):
                new_file = six.BytesIO()
                image.save(new_file, format)
                filename = self._save(filename, new_file)

        if file:
            file.close()

        content = {
            'filename': filename,
            'width': width,
            'height': height,
            'id': data.get('id') or None,
            'class': data.get('class') or None,
            'alt': data.get('alt') or None
        }

        return json.dumps(content)

    def delete(self, data):
        raise NotImplementedError

    def render(self, data):
        attrs = {
            'src': 'http://placekitten.com/160/90',
            'width': 160,
            'height': 90
        }
        if data:
            url = data.get('url')
            width = data.get('width') or 0
            height = data.get('height') or 0
            alt = data.get('alt')
            tag_id = data.get('id')
            tag_class = data.get('class')
            if url:
                attrs['src'] = url
            if alt:
                attrs['alt'] = alt
            if width and height:
                attrs['width'] = width
                attrs['height'] = height
            if tag_id:
                attrs['id'] = tag_id
            if tag_class:
                attrs['class'] = tag_class

        html_attrs = (u'{0}="{1}"'.format(attr, escape(attrs[attr])) for attr in sorted(attrs.keys()))
        return u'<img {0} />'.format(u' '.join(html_attrs))


class ImagePlugin(ImagePluginBase):
    """
    Image plugin extending abstract content-io image plugin to use django file storage.
    """
    @property
    def _file_storage(self):
        # Get configured file storage from settings
        file_storage = self.settings.get('FILE_STORAGE')

        # Fallback on default file storage
        if not file_storage:
            from django.core.files.storage import default_storage as file_storage

        return file_storage

    def _open(self, filename):
        return self._file_storage.open(filename)

    def _save(self, filename, bytes):
        content = InMemoryUploadedFile(bytes, None, filename, None, None, None)
        return self._file_storage.save(filename, content)

    def _url(self, filename):
        return self._file_storage.url(filename)



# coding=utf-8
VERSION = (1, 1, 3, 'final', 0)


def get_version(version=None):
    """Derives a PEP386-compliant version number from VERSION."""
    if version is None:
        version = VERSION
    assert len(version) == 5
    assert version[3] in ('alpha', 'beta', 'rc', 'final')

    # Now build the two parts of the version number:
    # main = X.Y[.Z]
    # sub = .devN - for pre-alpha releases
    #     | {a|b|c}N - for alpha, beta and rc releases

    parts = 2 if version[2] == 0 else 3
    main = '.'.join(str(x) for x in version[:parts])

    sub = ''
    if version[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[version[3]] + str(version[4])

    return main + sub


__version__ = get_version()


def configure():
    from django.conf import settings as django_settings
    from cio.conf import settings

    # Djedi default config
    config = dict(
        ENVIRONMENT={
            'default': {
                'i18n': django_settings.LANGUAGE_CODE,
                'l10n': getattr(django_settings, 'ROOT_URLCONF', 'local').split('.', 1)[0],
                'g11n': 'global'
            }
        },
        CACHE='djedi.backends.django.cache.Backend',
        STORAGE='djedi.backends.django.db.Backend',
        PLUGINS=[
            'cio.plugins.txt.TextPlugin',
            'cio.plugins.md.MarkdownPlugin',
            'djedi.plugins.img.ImagePlugin'
        ],
        THEME='darth'
    )

    # Update config with global djedi django settings
    config.update(getattr(django_settings, 'DJEDI', {}))

    # Overwrite config with prefixed variables from django settings
    for setting in ('ENVIRONMENT', 'CACHE', 'STORAGE', 'PIPELINE', 'PLUGINS', 'THEME'):
        conf = getattr(django_settings, 'DJEDI_%s' % setting, None)
        if conf is not None:
            config[setting] = conf

    # Configure content-io settings
    settings.configure(config)



import cio
from django.contrib.auth.models import User
from django.template import Context, TemplateSyntaxError
from cio.backends import cache
from cio.pipeline import pipeline
from djedi.templatetags.template import register
from djedi.tests.base import DjediTest, AssertionMixin

try:
    from django.template.loader import get_template_from_string
except ImportError:
    # Django 1.8+ has multiple template engines, we only test Django's for now.
    from django.template import engines

    def get_template_from_string(template_code):
        return engines['django'].from_string(template_code)


class TagTest(DjediTest, AssertionMixin):

    def render(self, source, context=None):
        source = u'{% load djedi_tags %}' + source.strip()
        context = Context(context or {})
        return get_template_from_string(source).render(context).strip()

    def test_node_tag(self):
        html = self.render(u"{% node 'page/title' edit=False %}")
        assert html == u''

        cio.set('i18n://sv-se@page/title.txt', u'Djedi')
        cio.set('i18n://sv-se@page/body.txt', u'Lightning fast!')

        with self.assertCache(calls=1, misses=0):
            with self.assertDB(calls=0):
                html = self.render(u"<h1>{% node 'page/title' edit=False %}</h1><p>{% node 'page/body' edit=False %}</p>")
                assert html == u'<h1>Djedi</h1><p>Lightning fast!</p>'

        cache.clear()

        with self.assertCache(calls=1, misses=2):
            with self.assertDB(calls=1):
                html = self.render(u"<h1>{% node 'page/title' edit=False %}</h1><p>{% node 'page/body' %}</p>")
                assert html == u'<h1>Djedi</h1><p><span data-i18n="sv-se@page/body">Lightning fast!</span></p>'

        html = self.render(u"{% node 'foo/bar' default='bogus' %}")
        assert html == u'<span data-i18n="sv-se@foo/bar">bogus</span>'
        html = self.render(u"{% node 'l10n://foo/bar' default='bogus' %}")
        self.assertEqual(html, u'<span data-i18n="djedi@foo/bar">bogus</span>')

    def test_blocknode_tag(self):
        with self.assertRaises(TemplateSyntaxError):
            self.render("{% blocknode 'page/body' arg %}{% endblocknode %}")

        html = self.render(u"""
            {% blocknode 'page/body.md' edit=False %}
                # Djedi
                Lightning *fast*!
            {% endblocknode %}
        """)
        self.assertRenderedMarkdown(html, u'# Djedi\nLightning *fast*!')

        cio.set('i18n://sv-se@page/body.txt', u'Lightning fast!')
        html = self.render(u"""
            {% blocknode "page/body" %}
                Lorem ipsum
            {% endblocknode %}
        """)
        assert html == u'<span data-i18n="sv-se@page/body">Lightning fast!</span>'

        cio.set('i18n://sv-se@page/body.txt', u'')
        html = self.render(u"{% blocknode 'page/body' edit=False %}Lorem ipsum{% endblocknode %}")
        assert html == u''

    def test_blocknode_with_context(self):
        cio.set('i18n://sv-se@page/title.txt', u'Hej {name}!')

        source = u"""
            {% blocknode 'page/title' edit=False name=user.get_full_name %}
                Hello {name}!
            {% endblocknode %}
        """

        context = {'user': User(first_name=u'Jonas', last_name=u'Lundberg')}
        html = self.render(source, context)
        assert html == u'Hej Jonas Lundberg!'

        with cio.env(i18n='en-us'):
            html = self.render(source, context)
            assert html == u'Hello Jonas Lundberg!'

        html = self.render(u"""
            {% blocknode 'page/title' edit=False %}
                Hello {name}!
            {% endblocknode %}
        """)
        assert html == u'Hej {name}!'

    def test_collected_nodes(self):
        source = u"""
            {% node 'page/title' edit=False %}
            {% node 'page/title' default='fallback' edit=False %}
            {% node 'page/body' edit=False %}
        """
        pipeline.history.clear()
        self.render(source)
        assert len(pipeline.history) == 2

    def test_invalid_lazy_tag(self):
        with self.assertRaises(TemplateSyntaxError):
            register.lazy_tag('')

    def test_lazy_tag(self):
        @register.lazy_tag
        def foo():
            return lambda _: u'bar'
        html = self.render("{% foo %}")
        assert html == u'bar'

        @register.lazy_tag()
        def bar():
            return lambda _: u'foo'
        html = self.render("{% bar %}")
        assert html == u'foo'

    def test_djedi_admin_tag(self):
        source = u"""
            {% load djedi_admin %}
            {% djedi_admin %}
        """

        user = User(first_name=u'Jonas', last_name=u'Lundberg')
        context = {'user': user}
        html = self.render(source, context)
        assert html == u''

        user.is_superuser = True
        html = self.render(source, context)
        assert u'<script>window.DJEDI_NODES = {};</script>' in html



from collections import defaultdict
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, Http404, HttpResponseBadRequest
from django.utils.http import urlunquote
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

import cio
from cio.plugins import plugins
from cio.plugins.exceptions import UnknownPlugin
from cio.utils.uri import URI

from .exceptions import InvalidNodeData
from .mixins import JSONResponseMixin, DjediContextMixin
from ..compat import TemplateResponse
from .. import auth


class APIView(View):

    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        if not auth.has_permission(request.user):
            raise PermissionDenied

        try:
            return super(APIView, self).dispatch(request, *args, **kwargs)
        except Http404:
            raise
        except Exception as e:
            return HttpResponseBadRequest(e)

    def get_post_data(self, request):
        """
        Collect and merge post parameters with multipart files.
        """
        params = dict(request.POST)
        params.update(request.FILES)

        data = defaultdict(dict)

        # Split data and meta parameters
        for param in sorted(params.keys()):
            value = params[param]

            if isinstance(value, list) and len(value) <= 1:
                value = value[0] if value else None

            prefix, _, field = param.partition('[')
            if field:
                field = field[:-1]
                try:
                    data[prefix][field] = value
                except TypeError:
                    raise InvalidNodeData('Got both reserved parameter "data" and plugin specific parameters.')
            else:
                data[prefix] = value

        return data['data'], data['meta']

    def decode_uri(self, uri):
        decoded = urlunquote(uri)

        # If uri got decoded then recursive try more times until nothing more can be decoded
        if decoded != uri:
            decoded = self.decode_uri(decoded)

        return decoded

    def render_to_response(self, content=u''):
        return HttpResponse(content)


class NodeApi(JSONResponseMixin, APIView):

    @never_cache
    def get(self, request, uri):
        """
        Return published node or specified version.

        JSON Response:
            {uri: x, content: y}
        """
        uri = self.decode_uri(uri)
        node = cio.get(uri, lazy=False)

        if node.content is None:
            raise Http404

        return self.render_to_json({
            'uri': node.uri,
            'content': node.content
        })

    def post(self, request, uri):
        """
        Set node data for uri, return rendered content.

        JSON Response:
            {uri: x, content: y}
        """
        uri = self.decode_uri(uri)
        data, meta = self.get_post_data(request)
        meta['author'] = auth.get_username(request.user)
        node = cio.set(uri, data, publish=False, **meta)
        return self.render_to_json(node)

    def delete(self, request, uri):
        """
        Delete versioned uri and return empty text response on success.
        """
        uri = self.decode_uri(uri)
        uris = cio.delete(uri)

        if uri not in uris:
            raise Http404

        return self.render_to_response()


class PublishApi(JSONResponseMixin, APIView):

    def put(self, request, uri):
        """
        Publish versioned uri.

        JSON Response:
            {uri: x, content: y}
        """
        uri = self.decode_uri(uri)
        node = cio.publish(uri)

        if not node:
            raise Http404

        return self.render_to_json(node)


class RevisionsApi(JSONResponseMixin, APIView):

    def get(self, request, uri):
        """
        List uri revisions.

        JSON Response:
            [[uri, state], ...]
        """
        uri = self.decode_uri(uri)
        revisions = cio.revisions(uri)
        revisions = [list(revision) for revision in revisions]  # Convert tuples to lists
        return self.render_to_json(revisions)


class LoadApi(JSONResponseMixin, APIView):

    @never_cache
    def get(self, request, uri):
        """
        Load raw node source from storage.

        JSON Response:
            {uri: x, data: y}
        """
        uri = self.decode_uri(uri)
        node = cio.load(uri)
        return self.render_to_json(node)


class RenderApi(APIView):

    def post(self, request, ext):
        """
        Render data for plugin and return text response.
        """
        try:
            plugin = plugins.get(ext)
            data, meta = self.get_post_data(request)
            data = plugin.load(data)
        except UnknownPlugin:
            raise Http404
        else:
            content = plugin.render(data)
            return self.render_to_response(content)


class NodeEditor(JSONResponseMixin, DjediContextMixin, APIView):

    @never_cache
    def get(self, request, uri):
        try:
            uri = self.decode_uri(uri)
            uri = URI(uri)
            plugins.resolve(uri)
        except UnknownPlugin:
            raise Http404
        else:
            return self.render_plugin(request, self.get_context_data(uri=uri))

    @never_cache
    def post(self, request, uri):
        uri = self.decode_uri(uri)
        data, meta = self.get_post_data(request)
        meta['author'] = auth.get_username(request.user)
        node = cio.set(uri, data, publish=False, **meta)

        context = cio.load(node.uri)
        context['content'] = node.content

        if request.is_ajax():
            return self.render_to_json(context)
        else:
            return self.render_plugin(request, context)

    def render_plugin(self, request, context):
        return TemplateResponse(request, [
            'djedi/plugins/%s/editor.html' % context['uri'].ext,
            'djedi/plugins/base/editor.html'
        ], self.get_context_data(**context))



import json

import cio

from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils import translation

from cio.conf import settings
from cio.pipeline import pipeline
from djedi.auth import has_permission
from djedi.compat import render_to_string


class TranslationMixin(object):

    def activate_language(self):
        # Activate current django translation
        language = translation.get_language()
        cio.env.push_state(i18n=language)


class AdminPanelMixin(object):

    def inject_admin_panel(self, request, response):
        user = getattr(request, 'user', None)

        # Validate user permissions
        if not has_permission(user):
            return

        # Do not inject admin panel in admin
        try:
            djedi_cms_url = reverse('admin:djedi:cms')
        except NoReverseMatch:
            raise ImproperlyConfigured('Could not find djedi in your url conf, '
                                       'enable django admin or include '
                                       'djedi.urls within the admin namespace.')
        else:
            admin_prefix = djedi_cms_url.strip('/').split('/')[0]
            if request.path.startswith('/' + admin_prefix):
                return

        # Do not inject admin panel on gzipped responses
        if 'gzip' in response.get('Content-Encoding', ''):
            return

        # Only inject admin panel in html pages
        content_type = response.get('Content-Type', '').split(';')[0]
        if content_type not in ('text/html', 'application/xhtml+xml'):
            return

        embed = self.render_cms()
        self.body_append(response, embed)

    def render_cms(self):
        def get_requested_uri(node):
            # Get first namespace URI, remove any version and ensures extension.
            # TODO: Default extension fallback should be handled in content-io
            uri = node.namespace_uri
            uri = uri.clone(
                ext=uri.ext or settings.URI_DEFAULT_EXT,
                version=None,
            )
            return uri

        defaults = dict(
            (get_requested_uri(node), node.initial)
            for node in pipeline.history.list('get')
        )

        return render_to_string('djedi/cms/embed.html', {
            'json_nodes': json.dumps(defaults).replace('</', '\\x3C/'),
        })

    def body_append(self, response, html):
        idx = response.content.lower().rfind(b'</body>')

        if idx >= 0:
            response.content = b''.join((response.content[:idx],
                                         html.encode('utf8'),
                                         response.content[idx:]))

            if response.get('Content-Length', None):
                response['Content-Length'] = len(response.content)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Node',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(max_length=255, db_index=True)),
                ('content', models.TextField(blank=True)),
                ('plugin', models.CharField(max_length=8)),
                ('version', models.CharField(max_length=255)),
                ('is_published', models.BooleanField(default=False)),
                ('meta', models.TextField(null=True, blank=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'db_table': 'djedi_node',
            },
            bases=(models.Model,),
        ),
    ]



try:
    from django.conf.urls import patterns, include
except ImportError:
    from django.conf.urls.defaults import patterns, include


urlpatterns = patterns('', (r'^', include('djedi.admin.urls', namespace='djedi', app_name='djedi')))



try:
    from django.conf.urls import patterns, include, url
except ImportError:
    from django.conf.urls.defaults import patterns, include, url

from django.contrib.admin import ModelAdmin
from django.core.exceptions import PermissionDenied
from django.views.generic import View

from .mixins import DjediContextMixin
from ..auth import has_permission
from ..compat import render


class Admin(ModelAdmin):

    verbose_name = 'CMS'
    verbose_name_plural = verbose_name

    def get_urls(self):
        return patterns(
            '',
            url(r'^', include('djedi.admin.urls', namespace='djedi')),
            url(r'', lambda: None, name='djedi_cms_changelist')  # Placeholder to show change link to CMS in admin
        )

    def has_change_permission(self, request, obj=None):
        return has_permission(request.user)

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class DjediCMS(DjediContextMixin, View):

    def get(self, request):
        if has_permission(request.user):
            return render(request, 'djedi/cms/cms.html', self.get_context_data())
        else:
            raise PermissionDenied



import django

if django.VERSION < (1, 5):
    from django.utils.encoding import smart_str, smart_unicode
else:
    from django.utils.encoding import smart_bytes as smart_str, smart_text as smart_unicode

__all__ = ['smart_str', 'smart_unicode']



from django.contrib import admin
from django.db.models import Model
from django.template.defaultfilters import pluralize
from djedi.admin import cms


def register(admin_class):
    name = admin_class.verbose_name
    name_plural = getattr(admin_class, 'verbose_name_plural', pluralize(name))

    model = type(name, (Model,), {
        '__module__': __name__,
        'Meta': type('Meta', (object,), dict(
            managed=False,
            abstract=True,
            app_label='djedi',
            verbose_name=name,
            verbose_name_plural=name_plural
        ))
    })

    admin.site._registry[model] = admin_class(model, admin.site)


register(cms.Admin)



from django.db import models


class Node(models.Model):

    key = models.CharField(max_length=255, db_index=True)
    content = models.TextField(blank=True)
    plugin = models.CharField(max_length=8)
    version = models.CharField(max_length=255)
    is_published = models.BooleanField(default=False, blank=True)
    meta = models.TextField(blank=True, null=True)
    date_created = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = u'djedi'
        db_table = 'djedi_node'



from cio.backends import cache
from djedi.tests.base import DjediTest


class CacheTest(DjediTest):

    def test_set(self):
        uri = 'i18n://sv-se@page/title.txt#1'
        content = u'Title'

        self.assertIsNone(cache.get(uri))
        cache.set(uri, content)

        node = cache.get(uri)
        self.assertEqual(node['uri'], uri)
        self.assertEqual(node['content'], content)

        cache.delete(uri)
        self.assertIsNone(cache.get(uri))



from cio.conf import settings
from cio.plugins import plugins
from djedi.tests import DjediTest


class SettingsTest(DjediTest):

    def test_settings(self):
        self.assertEqual(settings.THEME, 'luke')

    def test_plugin_settings(self):
        plugin = plugins.get('img')
        self.assertIn('foo', plugin.settings.keys())



from functools import partial
from inspect import getargspec
from django import template
from django.template import Context
from django.template.base import Node, TemplateSyntaxError, generic_tag_compiler

register = template.Library()


def lazy_tag(self, func=None, takes_context=None, name=None, node_class=None):
    """
    A tag function decorator, injected on Django's template tag library, similar to simple_tag().
    The decorated function gets called when the template node tree is built and should return
    another function, responsible for the output, that later will be called within the rendering phase.

    Note: if decorated with takes_context=True, context will not be available in the init phase.

    @register.lazy_tag(takes_context=True)
    def x(context, a, b, c=True, d=False):
        # Init phase (no context)

        def render(context):
            # Render phase
            return u'Content of argument a: %s' % a

        return render
    """
    def dec(func):
        params, varargs, varkw, defaults = getargspec(func)

        class SimpleNode(Node):

            def __init__(self, takes_context, args, kwargs):
                self.takes_context = takes_context
                self.args = args
                self.kwargs = kwargs

                resolved_args, resolved_kwargs = self.get_resolved_arguments(Context({}))

                self.resolved_args = resolved_args
                self.resolved_kwargs = resolved_kwargs
                self.render_func = func(*resolved_args, **resolved_kwargs)

            def get_resolved_arguments(self, context):
                resolved_args = [var.resolve(context) for var in self.args]
                if self.takes_context:
                    resolved_args = [context] + resolved_args
                resolved_kwargs = dict((k, v.resolve(context)) for k, v in self.kwargs.items())
                return resolved_args, resolved_kwargs

            def render(self, context):
                return self.render_func(context)

        function_name = (name or
                         getattr(func, '_decorated_function', func).__name__)
        compile_func = partial(generic_tag_compiler,
                               params=params, varargs=varargs, varkw=varkw,
                               defaults=defaults, name=function_name,
                               takes_context=takes_context, node_class=node_class or SimpleNode)
        compile_func.__doc__ = func.__doc__
        self.tag(function_name, compile_func)

        return func

    if func is None:
        return dec        # @register.lazy_tag(...)
    elif callable(func):
        return dec(func)  # @register.lazy_tag
    else:
        raise TemplateSyntaxError("Invalid arguments provided to lazy_tag")


template.Library.lazy_tag = lazy_tag



from django.core.urlresolvers import reverse
from djedi.utils.encoding import smart_unicode
from djedi.tests.base import ClientTest


class PanelTest(ClientTest):

    def test_embed(self):
        url = reverse('index')
        response = self.client.get(url)
        self.assertIn(u'Djedi Test', smart_unicode(response.content))
        self.assertIn(u'window.DJEDI_NODES', smart_unicode(response.content))
        self.assertIn(u'i18n://sv-se@foo/bar.txt', smart_unicode(response.content))
        self.assertIn(u'</body>', smart_unicode(response.content).lower())

    def test_cms(self):
        url = reverse('admin:djedi:cms')
        response = self.client.get(url)
        self.assertIn(u'<title>djedi cms</title>', smart_unicode(response.content))

    def test_django_admin(self):
        # Patch django admin index
        from django.contrib.admin.templatetags.log import AdminLogNode
        _render = AdminLogNode.render
        AdminLogNode.render = lambda x, y: None

        url = reverse('admin:index')
        response = self.client.get(url)
        cms_url = reverse('admin:djedi:cms')
        self.assertIn(u'<a href="%s">CMS</a>' % cms_url, smart_unicode(response.content))

        # Rollback patch
        AdminLogNode.render = _render



import cio
from cio.pipeline import pipeline


class DjediMiddleware(object):

    def process_request(self, request):
        # Bootstrap content-io
        pipeline.clear()
        cio.env.reset()

    def process_response(self, request, response):
        pipeline.clear()
        return response

    def process_exception(self, request, exception):
        pipeline.clear()



def has_permission(user):
    if user:
        if user.is_superuser:
            return True

        if user.is_staff and user.groups.filter(name__iexact='djedi').exists():
            return True

    return False


def get_username(user):
    if hasattr(user, 'get_username'):
        return user.get_username()
    else:
        return user.username



from .admin import DjediAdminMiddleware
from .mixins import TranslationMixin


class DjediTranslationMiddleware(DjediAdminMiddleware, TranslationMixin):

    def process_request(self, request):
        super(DjediTranslationMiddleware, self).process_request(request)
        self.activate_language()



#!/usr/bin/env python
from setuptools import setup, find_packages
from sys import version_info


install_requires = [
    'six',
    'content-io >= 1.2.2',
    'simplejson >= 3.2.0'
]

tests_require = [
    'coverage',
    'Markdown <= 2.4.1',
    'Pillow',
]

if version_info < (3,):
    tests_require += ['unittest2']

version = __import__('djedi').__version__

setup(
    name='djedi-cms',
    version=version,
    description='Django content management as it should be',
    long_description=(
        '.. image:: https://raw.github.com/5monkeys/djedi-cms/master/docs/_static/djedi-portrait.png\n\n'
        '- Read the documentation_\n'
        '- Browse the source_\n\n'
        '.. image:: https://travis-ci.org/5monkeys/djedi-cms.png?branch=master\n'
        '    :target: https://travis-ci.org/5monkeys/djedi-cms\n'
        '.. image:: https://coveralls.io/repos/5monkeys/djedi-cms/badge.png?branch=master\n'
        '    :target: https://coveralls.io/r/5monkeys/djedi-cms?branch=master\n\n'
        '.. _documentation: http://djedi-cms.org/\n'
        '.. _source: https://github.com/5monkeys/djedi-cms\n\n'
    ),
    author='Jonas Lundberg',
    author_email='jonas@5monkeys.se',
    url='https://github.com/5monkeys/djedi-cms',
    download_url='https://github.com/5monkeys/djedi-cms/tarball/%s' % version,
    keywords=['cms', 'django', 'edit', 'gettext', 'content', 'management', 'template', 'plugins', 'markdown'],
    license='BSD',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=install_requires,
    extras_require={
        'tests': tests_require,
    },
    tests_require=tests_require,
    test_suite='runtests.main'
)



import django

from functools import partial

from django.shortcuts import render
from django.template.loader import render_to_string
from django.template.response import TemplateResponse as BaseTemplateResponse

__all__ = ['render_to_string', 'render']

if django.VERSION >= (1, 8):
    # Always use the Django template engine on Django 1.8.
    render_to_string = partial(render_to_string, using='django')
    render = partial(render, using='django')

    class TemplateResponse(BaseTemplateResponse):
        def __init__(self, *args, **kwargs):
            kwargs['using'] = 'django'
            super(TemplateResponse, self).__init__(*args, **kwargs)
else:
    TemplateResponse = BaseTemplateResponse
